/**
 * JASMINe Deploy ME [Managed Element]
 * Copyright (C) 2012 Bull S.A.S.
 * Copyright (C) 2012 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Tomcat7Extension.java 10000 2012-05-02 14:52:40Z cazauxj $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.deployme.extensions.web.tomcat.tomcat7;

import org.ow2.jasmine.deployme.api.DeployMEPropertiesManager;
import org.ow2.jasmine.deployme.api.DeploymeVersion;
import org.ow2.jasmine.deployme.api.extensions.DeploymeExtensionException;
import org.ow2.jasmine.deployme.extension.AbstractDeploymeExtension;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7Type;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7ConnectorsType;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7AjpType;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7HttpType;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7HttpsType;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7SessionReplicationType;
import org.ow2.jasmine.deployme.extension.web.tomcat.tomcat7.generated.Tomcat7SessionManagerType;
import org.ow2.jasmine.deployme.v2.generated.OndemandType;
import org.ow2.jasmine.deployme.v2.generated.TopologyType;
import org.ow2.jonas.tools.configurator.api.JonasConfigurator;
import org.w3c.dom.Node;

import javax.xml.transform.Source;
import java.net.URL;
import java.util.List;

/**
 * Tomcat 7 extension
 * @author Jeremy Cazaux
 */
public class Tomcat7Extension extends AbstractDeploymeExtension {

    /**
     * Tomcat 7 namespace
     */
    public static String NAMESPACE = "http://jasmine.ow2.org/deployme/web-tomcat7-1.0";

    /**
     * Tomcat 7 XSD path
     */
    public static final String XSD_TOMCAT7 = "deployme-web-tomcat7.xsd";

    /**
     * Root class
     */
    public static Class<?> rootClass = Tomcat7Type.class;

    /**
     * Service represent by this extension
     */
    public static String SERVICE = "web";

    /**
     * Default constructor
     */
    public Tomcat7Extension() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    public Object convert(final Node localNode) throws DeploymeExtensionException {
        URL parentXSD = TopologyType.class.getClassLoader().getResource(
                DeployMEPropertiesManager.getXsdTopologyPath(DeploymeVersion.DEPLOYME_2));
        if (parentXSD == null) {
            throw new DeploymeExtensionException("Cannot get the URL of the topology XSD "  +
                    DeployMEPropertiesManager.getXsdTopologyPath(DeploymeVersion.DEPLOYME_2));
        }

        //be carefull to add parent XSD before child XSD
        this.xsdUrls.add(parentXSD);
        this.xsdUrls.add(getXsdURL(XSD_TOMCAT7, Tomcat7Type.class));

        //get XSD sources
        List<Source> sources = getSources(this.xsdUrls);

        //unmarshall the node
        return unmarshall(localNode, this.rootClass, sources, this.getClass().getClassLoader());
    }

    /**
     * {@inheritDoc}
     */
    public void applyConfiguration(final JonasConfigurator configurator, final Object object) {
        Tomcat7Type tomcat7Configuration = (Tomcat7Type) object;
        if (tomcat7Configuration != null) {
            configurator.setWebcontainer("tomcat7");
            final Tomcat7ConnectorsType connectors = tomcat7Configuration.getConnectors();
            if (connectors != null) {
                final Tomcat7AjpType ajp = connectors.getAjp();
                configurator.setAjpConnectorActivation(ajp != null);
                if (ajp != null) {
                    configurator.setAjpConnectorActivation(true);
                    final Long ajpPort = ajp.getPort();
                    if (ajpPort != null) {
                        configurator.setAjpPort(String.valueOf(ajpPort));
                    } 
                    final Long ajpAcceptCount = ajp.getAcceptCount();
                    if (ajpAcceptCount != null) {
                        configurator.setAjpAcceptCount(String.valueOf(ajpAcceptCount));
                    }
                    final Long ajpConnectionTimeout = ajp.getConnectionTimeout();
                    if (ajpConnectionTimeout != null) {
                        configurator.setAjpConnectionTimeout(String.valueOf(ajpConnectionTimeout));
                    }
                    final Long ajpMaxThreads = ajp.getMaxThreads();
                    if (ajpMaxThreads != null) {
                        configurator.setAjpMaxThreads(String.valueOf(ajpMaxThreads));
                    }
                    final Long ajpMinSpareThreads = ajp.getMinSpareThreads();
                    if (ajpMinSpareThreads != null) {
                        configurator.setAjpMinSpareThreads(String.valueOf(ajpMinSpareThreads));
                    }
                    final Long redirectPort = ajp.getRedirectPort();
                    if (redirectPort != null) {
                        configurator.setAjpRedirectPort(String.valueOf(redirectPort));
                    }
                    final Boolean isEnableLookups = ajp.isEnableLookups();
                    if (isEnableLookups != null) {
                        configurator.setAjpEnableLookups(isEnableLookups);
                    }
                }
                
                final Tomcat7HttpType http = connectors.getHttp();
                configurator.setHttpConnectorActivation(http != null);
                if (http != null) {
                    final Long httpPort = http.getPort();
                    if (httpPort != null) {
                        configurator.setHttpPort(String.valueOf(httpPort));
                    }
                    final Boolean isEnableLookups = http.isEnableLookups();
                    if (isEnableLookups != null) {
                        configurator.setHttpEnableLookups(isEnableLookups);   
                    }
                    final Long acceptCount = http.getAcceptCount();
                    if (acceptCount != null) {
                        configurator.setHttpAcceptCount(String.valueOf(acceptCount));
                    }
                    final String compression = http.getCompression();
                    if (compression != null) {
                        configurator.setHttpCompression(compression);
                    }
                    final Long connectionTimeout = http.getConnectionTimeout();
                    if (connectionTimeout != null) {
                        configurator.setHttpConnectionTimeout(String.valueOf(connectionTimeout));
                    }
                    final Long maxKeepAliveRequest = http.getMaxKeepAliveRequest();
                    if (maxKeepAliveRequest != null) {
                        configurator.setHttpMaxKeepAliveRequest(String.valueOf(maxKeepAliveRequest));
                    }
                    final Long maxThreads = http.getMaxThreads();
                    if (maxThreads != null) {
                        configurator.setHttpMaxThreads(String.valueOf(maxThreads));
                    }
                    final Long minSpareThreads = http.getMinSpareThreads();
                    if (minSpareThreads != null) {
                        configurator.setHttpMinSpareThreads(String.valueOf(minSpareThreads));
                    }
                    final Long redirectPort = http.getRedirectPort();
                    if (redirectPort != null) {
                        configurator.setHttpRedirectPort(String.valueOf(redirectPort));
                    }
                }
                
                final Tomcat7HttpsType https = connectors.getHttps();
                configurator.setHttpsConnectorActivation(https != null);
                if (https != null) {
                    final Long httpsPort = https.getPort();
                    if (httpsPort != null) {
                        configurator.setHttpsPort(String.valueOf(httpsPort));
                    }
                    final String httpsKeystoreFile = https.getKeystoreFile();
                    if (httpsKeystoreFile != null) {
                        configurator.setHttpsKeystoreFile(httpsKeystoreFile);
                    }
                    final String httpsKeystorePass = https.getKeystorePass();
                    if (httpsKeystorePass != null) {
                        configurator.setHttpsKeystorePass(httpsKeystorePass);
                    }
                    final Boolean isEnableLookups = https.isEnableLookups();
                    if (isEnableLookups != null) {
                        configurator.setHttpsEnableLookups(isEnableLookups);
                    }
                    final Long acceptCount = https.getAcceptCount();
                    if (acceptCount != null) {
                        configurator.setHttpsAcceptCount(String.valueOf(acceptCount));
                    }
                    final String compression = https.getCompression();
                    if (compression != null) {
                        configurator.setHttpsCompression(compression);
                    }
                    final Long connectionTimeout = https.getConnectionTimeout();
                    if (connectionTimeout != null) {
                        configurator.setHttpsConnectionTimeout(String.valueOf(connectionTimeout));
                    }
                    final Long maxKeepAliveRequest = https.getMaxKeepAliveRequest();
                    if (maxKeepAliveRequest != null) {
                        configurator.setHttpsMaxKeepAliveRequest(String.valueOf(maxKeepAliveRequest));
                    }
                    final Long maxThreads = https.getMaxThreads();
                    if (maxThreads != null) {
                        configurator.setHttpsMaxThreads(String.valueOf(maxThreads));
                    }
                    final Long minSpareThreads = https.getMinSpareThreads();
                    if (minSpareThreads != null) {
                        configurator.setHttpsMinSpareThreads(String.valueOf(minSpareThreads));
                    }
                    final Long redirectPort = https.getRedirectPort();
                    if (redirectPort != null) {
                        configurator.setHttpsRedirectPort(String.valueOf(redirectPort));
                    }
                }
                
                final OndemandType ondemand = tomcat7Configuration.getOndemand();
                configurator.setWebOndemandActivation(ondemand != null);
                if (ondemand != null) {
                    final Long webOndemandRedirectPort = ondemand.getRedirectPort();
                    if (webOndemandRedirectPort != null) {
                        configurator.setWebOndemandRedirectPort(String.valueOf(webOndemandRedirectPort));
                    }
                }

                final String jvmRoute = tomcat7Configuration.getJvmRoute();
                if (jvmRoute != null) {
                    configurator.setJvmRoute(jvmRoute);
                }

                final Tomcat7SessionReplicationType sessionReplication = tomcat7Configuration.getSessionReplication();
                configurator.setHttpSessionReplicationActivation(sessionReplication != null);
                if (sessionReplication != null) {
                    final String algorithm = sessionReplication.getAlgorithm();
                    if (algorithm != null) {
                        configurator.setHttpReplicationAlgorithm(algorithm);
                    }
                    final String clusterName = sessionReplication.getClusterName();
                    if (clusterName != null) {
                       configurator.setHttpReplicationClusterName(clusterName);
                    }
                    Long listenPort = sessionReplication.getListenPort();
                    if (listenPort != null) {
                        configurator.setHttpReplicationListenPort(String.valueOf(listenPort));
                    }
                    final String multicastAddress = sessionReplication.getMulticastAdress();
                    if (multicastAddress != null) {
                        configurator.setHttpReplicationMulticastAddress(multicastAddress);
                    }
                    final Long multicastPort = sessionReplication.getMulticastPort();
                    if (multicastPort != null) {
                        configurator.setHttpReplicationMulticastPort(String.valueOf(multicastPort));
                    }
                }
                
                final Tomcat7SessionManagerType tomcat7SessionManager = tomcat7Configuration.getSessionManager();
                configurator.setHttpSessionManagerActivation(tomcat7SessionManager != null);
                if (tomcat7SessionManager != null) {
                    final Long maxActiveSession = tomcat7SessionManager.getMaxActiveSessions();
                    if (maxActiveSession != null) {
                        configurator.setMaxActiveWebSessions(String.valueOf(maxActiveSession));
                    }                      
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public String getNamespace() {
        return NAMESPACE;
    }

    /**
     * {@inheritDoc} 
     */
    public String getJOnASService() {
        return SERVICE;
    }
}
